import json


class PyRoyaleException(Exception):
    """Base Exception class for PyRoyale"""

    pass


class HTTPException(PyRoyaleException):
    def __init__(self, response_body, error_message: str) -> None:
        Exception.__init__(self)
        self._status_code = response_body.status
        self._error_message = json.loads(error_message)

    def __str__(self) -> str:
        return f"Status Code: {self._status_code} | {self._error_message['reason']} {self._error_message['message']}"


class BadRequest(HTTPException):
    """Exception that is thrown when HTTP Status Code 400 is returned"""

    pass


class Forbidden(HTTPException):
    """Exception that is thrown when HTTP Status Code 403 is returned"""

    pass


class NotFound(HTTPException):
    """Exception that is thrown when HTTP Status Code 404 is returned"""

    pass


class UnknownRequest(HTTPException):
    """Exception that is thrown when HTTP Status Code 500 is returned"""

    pass


class ServiceUnavailable(HTTPException):
    """Exception that is thrown when HTTP Status Code 503 is returned"""

    pass


class Removed(HTTPException):
    """Exception that is thrown when HTTP Status Code 410 is returned"""

    pass


class BadMethod(HTTPException):
    """Exception that is thrown when an invalid request type is given, something other than
    GET or POST
    """

    pass
